@@ -28,6 +28,7 @@ import android.content.Context;
 import android.content.Intent;
 import android.content.IntentFilter;
 import android.content.res.Configuration;
+import android.graphics.Bitmap;
 import android.graphics.PixelFormat;
 import android.hardware.display.DisplayManager;
 import android.media.AudioManager;
@@ -72,7 +73,7 @@ import butterknife.BindView;
 import butterknife.ButterKnife;
 import butterknife.OnClick;
 import cx.ring.R;
-import cx.ring.adapters.ContactPictureTask;
+import cx.ring.adapters.ContactDetailsTask;
 import cx.ring.client.ConversationActivity;
 import cx.ring.client.HomeActivity;
 import cx.ring.interfaces.CallInterface;
@@ -84,13 +85,15 @@ import cx.ring.service.CallManagerCallBack;
 import cx.ring.service.DRingService;
 import cx.ring.service.IDRingService;
 import cx.ring.service.LocalService;
-import cx.ring.utils.CropImageUtils;
+import cx.ring.utils.ActionHelper;
+import cx.ring.utils.ContentUriHandler;
+import cx.ring.utils.BitmapUtils;
 import cx.ring.utils.KeyboardVisibilityManager;
 import cx.ring.utils.VCardUtils;
 import ezvcard.VCard;
 import ezvcard.property.Photo;
 
-public class CallFragment extends Fragment implements CallInterface {
+public class CallFragment extends Fragment implements CallInterface, ContactDetailsTask.DetailsLoadedCallback {
 
     static final private String TAG = CallFragment.class.getSimpleName();
 
@@ -429,15 +432,17 @@ public class CallFragment extends Fragment implements CallInterface {
                         .setClass(getActivity(), ConversationActivity.class)
                         .setAction(Intent.ACTION_VIEW)
                         .setFlags(Intent.FLAG_ACTIVITY_CLEAR_TOP | Intent.FLAG_ACTIVITY_SINGLE_TOP)
-                        .setData(Uri.withAppendedPath(ConversationActivity.CONTENT_URI, firstParticipant.getContact().getIds().get(0)));
+                        .setData(Uri.withAppendedPath(ContentUriHandler.CONVERSATION_CONTENT_URI, firstParticipant.getContact().getIds().get(0)));
                 intent.putExtra("resuming", true);
                 startActivityForResult(intent, HomeActivity.REQUEST_CODE_CONVERSATION);
+
                 break;
             case R.id.menuitem_addcontact:
                 if (firstParticipant == null || firstParticipant.getContact() == null) {
                     break;
                 }
-                startActivityForResult(firstParticipant.getContact().getAddNumberIntent(), ConversationActivity.REQ_ADD_CONTACT);
+                startActivityForResult(ActionHelper.getAddNumberIntentForContact(firstParticipant.getContact()),
+                        ConversationActivity.REQ_ADD_CONTACT);
                 break;
             case R.id.menuitem_speaker:
                 audioManager.setSpeakerphoneOn(!audioManager.isSpeakerphoneOn());
@@ -450,7 +455,7 @@ public class CallFragment extends Fragment implements CallInterface {
                 } catch (RemoteException e) {
                     e.printStackTrace();
                 }
-                item.setIcon(lastVideoSource ? R.drawable.ic_camera_front_white_24dp : R.drawable.ic_camera_rear_white_24dp);
+                item.setIcon(lastVideoSource ? R.drawable.ic_camera_front_white : R.drawable.ic_camera_rear_white);
                 break;
             case R.id.menuitem_dialpad:
                 KeyboardVisibilityManager.showKeyboard(getActivity(),
@@ -496,12 +501,12 @@ public class CallFragment extends Fragment implements CallInterface {
         }
 
         Conference c = getConference();
-        if (c != null && mVideoSurface != null && c.resumeVideo) {
+        if (c != null && mVideoSurface != null && c.shouldResumeVideo()) {
             Log.i(TAG, "Resuming video");
             haveVideo = true;
             mVideoSurface.setVisibility(View.VISIBLE);
             videoPreview.setVisibility(View.VISIBLE);
-            c.resumeVideo = false;
+            c.setResumeVideo(false);
         }
     }
 
@@ -509,24 +514,24 @@ public class CallFragment extends Fragment implements CallInterface {
     public void onResume() {
         super.onResume();
         Log.i(TAG, "onResume()");
-        Conference c = getConference();
+        Conference conference = getConference();
 
-        this.confUpdate();
+        confUpdate();
 
         if (getActivity() != null) {
             getActivity().invalidateOptionsMenu();
         }
 
-        if (c != null) {
-            c.mVisible = true;
+        if (conference != null) {
+            conference.setVisible(true);
             NotificationManagerCompat notificationManager = NotificationManagerCompat.from(getActivity());
-            notificationManager.cancel(c.notificationId);
-            if (c.resumeVideo) {
+            notificationManager.cancel(conference.getUuid());
+            if (conference.shouldResumeVideo()) {
                 Log.w(TAG, "Resuming video");
                 haveVideo = true;
                 mVideoSurface.setVisibility(View.VISIBLE);
                 videoPreview.setVisibility(View.VISIBLE);
-                c.resumeVideo = false;
+                conference.setResumeVideo(false);
             }
         }
 
@@ -540,9 +545,9 @@ public class CallFragment extends Fragment implements CallInterface {
 
         Conference c = getConference();
         if (c != null) {
-            c.mVisible = false;
-            c.resumeVideo = haveVideo;
-            c.showCallNotification(getActivity());
+            c.setVisible(false);
+            c.setResumeVideo(haveVideo);
+            ActionHelper.showCallNotification(getActivity(), c);
         }
     }
 
@@ -560,7 +565,7 @@ public class CallFragment extends Fragment implements CallInterface {
             return;
         }
 
-        int newState = c.getHumanState();
+        int newState = getHumanState(c);
         String newStateString = (newState == R.string.call_human_state_none ||
                 newState == R.string.conference_human_state_default)
                 ? "" :
@@ -578,12 +583,66 @@ public class CallFragment extends Fragment implements CallInterface {
                 initOutGoingCallDisplay();
         } else {
             NotificationManagerCompat notificationManager = NotificationManagerCompat.from(getActivity());
-            notificationManager.cancel(c.notificationId);
+            notificationManager.cancel(c.getUuid());
             mCallStatusTxt.setText(newStateString);
             mCallbacks.terminateCall();
         }
     }
 
+    public int getHumanState(Conference conference) {
+        if (conference.getParticipants().size() == 1) {
+            return callStateToHumanState(conference.getParticipants().get(0).getCallState());
+        }
+        return getConferenceHumanState(conference.getState());
+    }
+
+    public static int callStateToHumanState(final int state) {
+        switch (state) {
+            case SipCall.State.INCOMING:
+                return R.string.call_human_state_incoming;
+            case SipCall.State.CONNECTING:
+                return R.string.call_human_state_connecting;
+            case SipCall.State.RINGING:
+                return R.string.call_human_state_ringing;
+            case SipCall.State.CURRENT:
+                return R.string.call_human_state_current;
+            case SipCall.State.HUNGUP:
+                return R.string.call_human_state_hungup;
+            case SipCall.State.BUSY:
+                return R.string.call_human_state_busy;
+            case SipCall.State.FAILURE:
+                return R.string.call_human_state_failure;
+            case SipCall.State.HOLD:
+                return R.string.call_human_state_hold;
+            case SipCall.State.UNHOLD:
+                return R.string.call_human_state_unhold;
+            case SipCall.State.OVER:
+                return R.string.call_human_state_over;
+            case SipCall.State.NONE:
+            default:
+                return R.string.call_human_state_none;
+        }
+    }
+
+    public int getConferenceHumanState(final int state) {
+        switch (state) {
+            case Conference.state.ACTIVE_ATTACHED:
+                return R.string.conference_human_state_active_attached;
+            case Conference.state.ACTIVE_DETACHED:
+                return R.string.conference_human_state_active_detached;
+            case Conference.state.ACTIVE_ATTACHED_REC:
+                return R.string.conference_human_state_active_attached_rec;
+            case Conference.state.ACTIVE_DETACHED_REC:
+                return R.string.conference_human_state_active_detached_rec;
+            case Conference.state.HOLD:
+                return R.string.conference_human_state_hold;
+            case Conference.state.HOLD_REC:
+                return R.string.conference_human_state_hold_rec;
+            default:
+                return R.string.conference_human_state_default;
+        }
+    }
+
     @Override
     public void recordingChanged(Conference c, String callID, String filename) {
 
@@ -668,12 +727,16 @@ public class CallFragment extends Fragment implements CallInterface {
 
     @Override
     public void onConfigurationChanged(Configuration newConfig) {
+        updatePreview();
+    }
+
+    private void updatePreview() {
         if (videoPreview.getVisibility() == View.VISIBLE) {
             try {
                 mCallbacks.getRemoteService().setPreviewSettings();
                 mCallbacks.getRemoteService().videoPreviewSurfaceAdded();
             } catch (RemoteException e) {
-                e.printStackTrace();
+                Log.e(TAG, "service not found ", e);
             }
         }
     }
@@ -823,10 +886,6 @@ public class CallFragment extends Fragment implements CallInterface {
         mPulseAnimation.startRippleAnimation();
 
         updateContactBubble();
-
-        ActionBar ab = mCallbacks.getSupportActionBar();
-        ab.setDisplayOptions(ActionBar.DISPLAY_SHOW_HOME | ActionBar.DISPLAY_HOME_AS_UP | ActionBar.DISPLAY_SHOW_TITLE);
-        ab.setTitle(name);
     }
 
     private void initNormalStateDisplay() {
@@ -845,6 +904,8 @@ public class CallFragment extends Fragment implements CallInterface {
 
         contactBubbleLayout.setVisibility(haveVideo ? View.GONE : View.VISIBLE);
         updateSecurityDisplay();
+
+        updatePreview();
     }
 
     private void updateSecurityDisplay() {
@@ -938,8 +999,9 @@ public class CallFragment extends Fragment implements CallInterface {
         }
 
         VCard vcard;
-        String username = participant.getNumberUri().username;
-        vcard = VCardUtils.loadFromDisk(username + ".vcf", context);
+        String username = participant.getNumberUri().getUsername();
+        Log.d(TAG, "username " + username);
+        vcard = VCardUtils.loadPeerProfileFromDisk(context.getFilesDir(), username + ".vcf");
         if (vcard == null) {
             Log.d(TAG, "No vcard.");
             setDefaultPhoto();
@@ -950,7 +1012,11 @@ public class CallFragment extends Fragment implements CallInterface {
 
         if (!vcard.getPhotos().isEmpty()) {
             Photo tmp = vcard.getPhotos().get(0);
-            contactBubbleView.setImageBitmap(CropImageUtils.cropImageToCircle(tmp.getData()));
+            if (tmp.getData() != null) {
+                contactBubbleView.setImageBitmap(BitmapUtils.cropImageToCircle(tmp.getData()));
+            } else {
+                setDefaultPhoto();
+            }
         } else {
             setDefaultPhoto();
         }
@@ -959,6 +1025,10 @@ public class CallFragment extends Fragment implements CallInterface {
             return;
         }
         contactBubbleTxt.setText(vcard.getFormattedName().getValue());
+        ActionBar ab = mCallbacks.getSupportActionBar();
+        ab.setDisplayOptions(ActionBar.DISPLAY_SHOW_HOME | ActionBar.DISPLAY_HOME_AS_UP | ActionBar.DISPLAY_SHOW_TITLE);
+        ab.setTitle(vcard.getFormattedName().getValue());
+
         if (participant.getNumber().contentEquals(vcard.getFormattedName().getValue())) {
             contactBubbleNumTxt.setVisibility(View.GONE);
         } else {
@@ -1016,11 +1086,25 @@ public class CallFragment extends Fragment implements CallInterface {
             final SipCall call = getConference().getParticipants().get(0);
             final CallContact contact = call.getContact();
             if (contact != null) {
-                new ContactPictureTask(getActivity(), contactBubbleView, contact).run();
+                new ContactDetailsTask(getActivity(), contact, this).run();
             }
         } else {
             contactBubbleView.setImageDrawable(
                     ResourcesCompat.getDrawable(getResources(), R.drawable.ic_contact_picture, null));
         }
     }
+
+    @Override
+    public void onDetailsLoaded(Bitmap bmp, String formattedName) {
+        if (bmp != null) {
+            contactBubbleView.setImageBitmap(bmp);
+        }
+
+        if (formattedName != null) {
+            contactBubbleTxt.setText(formattedName);
+            ActionBar ab = mCallbacks.getSupportActionBar();
+            ab.setDisplayOptions(ActionBar.DISPLAY_SHOW_HOME | ActionBar.DISPLAY_HOME_AS_UP | ActionBar.DISPLAY_SHOW_TITLE);
+            ab.setTitle(formattedName);
+        }
+    }
 }
