@@ -19,14 +19,6 @@
  */
 package cx.ring.fragments;
 
-import cx.ring.R;
-import cx.ring.model.account.AccountDetail;
-import cx.ring.model.account.AccountDetailBasic;
-import cx.ring.model.account.Account;
-import cx.ring.views.EditTextIntegerPreference;
-import cx.ring.views.EditTextPreferenceDialog;
-import cx.ring.views.PasswordPreference;
-
 import android.app.Activity;
 import android.os.Bundle;
 import android.support.v14.preference.PreferenceFragment;
@@ -36,41 +28,55 @@ import android.support.v7.preference.TwoStatePreference;
 import android.util.Log;
 import android.view.inputmethod.EditorInfo;
 
-public class GeneralAccountFragment extends PreferenceFragment {
+import cx.ring.R;
+import cx.ring.interfaces.AccountCallbacks;
+import cx.ring.interfaces.AccountChangedListener;
+import cx.ring.model.Account;
+import cx.ring.model.AccountConfig;
+import cx.ring.model.ConfigKey;
+import cx.ring.views.EditTextIntegerPreference;
+import cx.ring.views.EditTextPreferenceDialog;
+import cx.ring.views.PasswordPreference;
+
+import static cx.ring.client.AccountEditionActivity.DUMMY_CALLBACKS;
+
+public class GeneralAccountFragment extends PreferenceFragment implements AccountChangedListener {
 
     private static final String TAG = GeneralAccountFragment.class.getSimpleName();
     private static final String DIALOG_FRAGMENT_TAG = "android.support.v14.preference.PreferenceFragment.DIALOG";
-
-    private Callbacks mCallbacks = sDummyCallbacks;
-    private static final Callbacks sDummyCallbacks = new Callbacks() {
-        @Override
-        public Account getAccount() {
-            return null;
-        }
-    };
-
-    public interface Callbacks {
-        Account getAccount();
-    }
+    private static final String KEY_IS_RING = "accountIsRing";
+    private AccountCallbacks mCallbacks = DUMMY_CALLBACKS;
 
     @Override
     public void onAttach(Activity activity) {
         super.onAttach(activity);
-        if (!(activity instanceof Callbacks)) {
+        if (!(activity instanceof AccountCallbacks)) {
             throw new IllegalStateException("Activity must implement fragment's callbacks.");
         }
 
-        mCallbacks = (Callbacks) activity;
+        mCallbacks = (AccountCallbacks) activity;
+        mCallbacks.addOnAccountChanged(this);
     }
 
     @Override
     public void onDetach() {
         super.onDetach();
-        mCallbacks = sDummyCallbacks;
+        Log.i(TAG, "onDetach");
+        if (mCallbacks != null) {
+            mCallbacks.removeOnAccountChanged(this);
+        }
+        mCallbacks = DUMMY_CALLBACKS;
+    }
+
+    @Override
+    public void accountChanged(Account acc) {
+        setPreferenceDetails(acc.getConfig());
+        setPreferenceListener(acc.getConfig(), changeBasicPreferenceListener);
     }
 
     @Override
     public void onCreatePreferences(Bundle bundle, String s) {
+        Log.i(TAG, "onCreatePreferences " + bundle + " " + s);
         Account acc = mCallbacks.getAccount();
         if (acc != null) {
             if (acc.isRing()) {
@@ -78,8 +84,28 @@ public class GeneralAccountFragment extends PreferenceFragment {
             } else {
                 addPreferencesFromResource(R.xml.account_general_prefs);
             }
-            setPreferenceDetails(acc.getBasicDetails());
-            addPreferenceListener(acc.getBasicDetails(), changeBasicPreferenceListener);
+            accountChanged(acc);
+        } else {
+            if (bundle != null) {
+                Log.w(TAG, "onCreatePreferences: null account, from bundle");
+                boolean isRing = bundle.getBoolean(KEY_IS_RING);
+                if (isRing) {
+                    addPreferencesFromResource(R.xml.account_prefs_ring);
+                } else {
+                    addPreferencesFromResource(R.xml.account_general_prefs);
+                }
+            } else {
+                Log.w(TAG, "onCreatePreferences: null account");
+            }
+        }
+    }
+
+    @Override
+    public void onSaveInstanceState(Bundle outState) {
+        super.onSaveInstanceState(outState);
+        Account acc = mCallbacks.getAccount();
+        if (acc != null) {
+            outState.putBoolean(KEY_IS_RING, acc.isRing());
         }
     }
 
@@ -88,6 +114,7 @@ public class GeneralAccountFragment extends PreferenceFragment {
         if (getFragmentManager().findFragmentByTag(DIALOG_FRAGMENT_TAG) != null) {
             return;
         }
+
         if (preference instanceof EditTextIntegerPreference) {
             EditTextPreferenceDialog f = EditTextPreferenceDialog.newInstance(preference.getKey(), EditorInfo.TYPE_CLASS_NUMBER);
             f.setTargetFragment(this, 0);
@@ -101,69 +128,67 @@ public class GeneralAccountFragment extends PreferenceFragment {
         }
     }
 
-    private void setPreferenceDetails(AccountDetail details) {
-        for (AccountDetail.PreferenceEntry p : details.getDetailValues()) {
-            Preference pref = findPreference(p.mKey);
+    private void setPreferenceDetails(AccountConfig details) {
+        for (ConfigKey confKey : details.getKeys()) {
+            Preference pref = findPreference(confKey.key());
             if (pref != null) {
-                if (!p.isTwoState) {
-                    ((EditTextPreference) pref).setText(p.mValue);
+                if (!confKey.isTwoState()) {
+                    String val = details.get(confKey);
+                    ((EditTextPreference) pref).setText(val);
                     if (pref instanceof PasswordPreference) {
                         String tmp = "";
-                        for (int i = 0; i < p.mValue.length(); ++i) {
+                        for (int i = 0; i < val.length(); ++i) {
                             tmp += "*";
                         }
                         pref.setSummary(tmp);
                     } else {
-                        pref.setSummary(p.mValue);
+                        pref.setSummary(val);
                     }
                 } else {
-                    ((TwoStatePreference) pref).setChecked(p.isChecked());
+                    ((TwoStatePreference) pref).setChecked(details.getBool(confKey));
                 }
             }
         }
     }
 
-    private void addPreferenceListener(AccountDetail details, Preference.OnPreferenceChangeListener listener) {
-        for (AccountDetail.PreferenceEntry p : details.getDetailValues()) {
-            //Log.i(TAG, "addPreferenceListener: pref " + p.mKey + " " + p.mValue);
-            Preference pref = findPreference(p.mKey);
+    private void setPreferenceListener(AccountConfig details, Preference.OnPreferenceChangeListener listener) {
+        for (ConfigKey confKey : details.getKeys()) {
+            Preference pref = findPreference(confKey.key());
             if (pref != null) {
                 pref.setOnPreferenceChangeListener(listener);
-            } /*else {
-                Log.w(TAG, "addPreferenceListener: pref not found");
-            }*/
+            }
         }
     }
 
-    Preference.OnPreferenceChangeListener changeBasicPreferenceListener = new Preference.OnPreferenceChangeListener() {
+    private final Preference.OnPreferenceChangeListener changeBasicPreferenceListener = new Preference.OnPreferenceChangeListener() {
         @Override
         public boolean onPreferenceChange(Preference preference, Object newValue) {
-
             Log.i(TAG, "Changing preference " + preference.getKey() + " to value:" + newValue);
-            final Account acc = mCallbacks.getAccount();
+            final Account account = mCallbacks.getAccount();
+            final ConfigKey key = ConfigKey.fromString(preference.getKey());
             if (preference instanceof TwoStatePreference) {
-                acc.getBasicDetails().setDetailString(preference.getKey(), newValue.toString());
+                account.setDetail(key, newValue.toString());
             } else {
                 if (preference instanceof PasswordPreference) {
                     String tmp = "";
                     for (int i = 0; i < ((String) newValue).length(); ++i) {
                         tmp += "*";
                     }
-                    if(acc.isSip())
-                        acc.getCredentials().get(0).setDetailString(preference.getKey(), newValue.toString());
+                    if (account.isSip())
+                        account.getCredentials().get(0).setDetail(key, newValue.toString());
                     preference.setSummary(tmp);
-                } else if(preference.getKey().contentEquals(AccountDetailBasic.CONFIG_ACCOUNT_USERNAME)) {
-					if(acc.isSip()){
-                        acc.getCredentials().get(0).setDetailString(preference.getKey(), newValue.toString());
-					}
+                } else if (key == ConfigKey.ACCOUNT_USERNAME) {
+                    if (account.isSip()) {
+                        account.getCredentials().get(0).setDetail(key, newValue.toString());
+                    }
                     preference.setSummary((CharSequence) newValue);
                 } else {
                     preference.setSummary((CharSequence) newValue);
                 }
 
-                acc.getBasicDetails().setDetailString(preference.getKey(), newValue.toString());
+                account.setDetail(key, newValue.toString());
             }
-            acc.notifyObservers();
+            mCallbacks.saveAccount();
             return true;
         }
     };
