@@ -43,6 +43,7 @@
 #include "recentmodel.h"
 #include "contactmethod.h"
 #include "globalinstances.h"
+#include <availableaccountmodel.h>
 
 #include "wizarddialog.h"
 #include "windowscontactbackend.h"
@@ -172,19 +173,23 @@ CallWidget::CallWidget(QWidget* parent) :
         connect(ui->smartList, &SmartList::btnVideoClicked, this, &CallWidget::btnComBarVideoClicked);
 
         connect(RecentModel::instance().selectionModel(),
-                SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
+                SIGNAL(currentChanged(QModelIndex,QModelIndex)),
                 this,
-                SLOT(smartListSelectionChanged(QItemSelection,QItemSelection)));
+                SLOT(smartListCurrentChanged(QModelIndex,QModelIndex)));
 
         connect(RecentModel::instance().selectionModel(), &QItemSelectionModel::selectionChanged, [=](const QItemSelection &selected, const QItemSelection &deselected) {
-            Q_UNUSED(deselected)
-            if (selected.size()) {
-                auto idx = selected.indexes().first();
-                auto realIdx = RecentModel::instance().peopleProxy()->mapFromSource(idx);
-                ui->smartList->selectionModel()->setCurrentIndex(realIdx, QItemSelectionModel::ClearAndSelect);
-            } else
-                ui->smartList->clearSelection();
-        });
+                    // lambda used to focus on the correct smartList element when switching automatically between two calls
+                    Q_UNUSED(deselected)
+                    if (selected.size()) {
+                        auto idx = selected.indexes().first();
+                        auto realIdx = RecentModel::instance().peopleProxy()->mapFromSource(idx);
+                        ui->smartList->selectionModel()->setCurrentIndex(realIdx, QItemSelectionModel::ClearAndSelect);
+                    } else
+                        ui->smartList->clearSelection();
+                });
+
+        connect(&NameDirectory::instance(), SIGNAL(registeredNameFound(const Account*,NameDirectory::LookupStatus,const QString&,const QString&)),
+                this, SLOT(contactLineEdit_registeredNameFound(const Account*,NameDirectory::LookupStatus,const QString&,const QString&)));
 
     } catch (const std::exception& e) {
         qDebug() << "INIT ERROR" << e.what();
@@ -376,6 +381,8 @@ void CallWidget::setupQRCode()
 void
 CallWidget::findRingAccount()
 {
+    ui->ringIdLabel->clear();
+
     auto a_count = AccountModel::instance().rowCount();
     for (int i = 0; i < a_count; ++i) {
         auto idx = AccountModel::instance().index(i, 0);
@@ -384,16 +391,22 @@ CallWidget::findRingAccount()
             auto account = AccountModel::instance().getAccountByModelIndex(idx);
             if (account->displayName().isEmpty())
                 account->displayName() = account->alias();
-            auto username = account->username();
-            ui->ringIdLabel->setText(username);
-            setupQRCode();
-            return;
+            if (account->needsMigration()) {
+                WizardDialog dlg(WizardDialog::MIGRATION, account);
+                dlg.exec();
+            }
+            if (ui->ringIdLabel->text().isEmpty()) {
+                auto username = account->username();
+                ui->ringIdLabel->setText(username);
+                setupQRCode();
+            }
         }
     }
-    ui->ringIdLabel->setText(tr("NO RING ACCOUNT FOUND"));
-    auto wizardDialog = new WizardDialog();
-    wizardDialog->exec();
-    delete wizardDialog;
+    if (ui->ringIdLabel->text().isEmpty()) {
+        ui->ringIdLabel->setText(tr("NO RING ACCOUNT FOUND"));
+        WizardDialog wizardDialog;
+        wizardDialog.exec();
+    }
 }
 
 void
@@ -407,6 +420,12 @@ CallWidget::callChangedSlot()
                 QPixmap::fromImage(
                     GlobalInstances::pixmapManipulator()
                     .callPhoto(actualCall_, QSize(130,130)).value<QImage>()));
+
+    if(actualCall_->state() == Call::State::OVER || actualCall_->state() == Call::State::FAILURE){
+        ui->outboundCallLabel->setText(actualCall_->toHumanStateName(Call::State::INITIALIZATION));
+    } else {
+        ui->outboundCallLabel->setText(actualCall_->toHumanStateName());
+    }
 }
 
 void
@@ -447,11 +466,13 @@ CallWidget::callStateChanged(Call* call, Call::State previousState)
             || call != actualCall_)
         return;
 
+    callStateToView(call);
+
     if (call->state() == Call::State::OVER) {
         setActualCall(nullptr);
+        ui->smartList->clearSelection();
         RecentModel::instance().selectionModel()->clear();
     }
-    callStateToView(call);
 }
 
 void
@@ -521,11 +542,13 @@ void CallWidget::callStateToView(Call* value)
         case Call::State::OVER:
             ui->stackedWidget->setCurrentWidget(ui->welcomePage);
             break;
+        case Call::State::FAILURE:
+        case Call::State::ERROR:
+            on_cancelButton_clicked();
+            break;
         case Call::State::INITIALIZATION:
         case Call::State::CONNECTED:
         case Call::State::RINGING:
-        case Call::State::FAILURE:
-        case Call::State::ERROR:
             ui->stackedWidget->setCurrentWidget(ui->outboundCallPage);
             break;
         default:
@@ -557,8 +580,10 @@ CallWidget::setActualCall(Call* value)
 void
 CallWidget::on_cancelButton_clicked()
 {
-    if (actualCall_)
+    if (actualCall_){
         actualCall_->performAction(Call::Action::REFUSE);
+        ui->stackedWidget->setCurrentWidget(ui->welcomePage);
+    }
 }
 
 void
@@ -584,29 +609,22 @@ CallWidget::on_smartList_doubleClicked(const QModelIndex& index)
 }
 
 void
-CallWidget::smartListSelectionChanged(const QItemSelection& newSel, const QItemSelection& oldSel) {
-
-    Q_UNUSED(oldSel)
-
-    if (newSel.indexes().empty()) {
-        setActualCall(nullptr);
-        return ui->stackedWidget->setCurrentWidget(ui->welcomePage);
-    }
-
-    auto newIdx = newSel.indexes().first();
-    if (not newIdx.isValid())
+CallWidget::smartListCurrentChanged(const QModelIndex &currentIdx, const QModelIndex &previousIdx)
+{
+    Q_UNUSED(previousIdx);
+    if (not currentIdx.isValid())
         return;
 
-    auto newIdxCall = RecentModel::instance().getActiveCall(newIdx);
-    if (newIdxCall && newIdxCall != actualCall_) {
-        setActualCall(newIdxCall);
-    } else if (newIdxCall == nullptr){
+    //catch call of current index
+    auto currentIdxCall = RecentModel::instance().getActiveCall(currentIdx);
+
+    if (currentIdxCall && currentIdxCall != actualCall_) { //if it is different from actual call, switch between the two
+        setActualCall(currentIdxCall);
+    } else if (currentIdxCall == nullptr){ // if there is no call attached to this smartlist index (contact tab)
         setActualCall(nullptr);
-        ui->instantMessagingWidget->hide();
-        showIMOutOfCall(newIdx);
-    } else {
+        showIMOutOfCall(currentIdx); //change page to messaging page with correct behaviour
+    } else { // if non defined behaviour disconnect instant messaging and return to welcome page
         setActualCall(nullptr);
-        ui->instantMessagingWidget->hide();
         if (imConnection_)
             disconnect(imConnection_);
         ui->stackedWidget->setCurrentWidget(ui->welcomePage);
@@ -645,17 +663,49 @@ CallWidget::historicButtonClicked(bool checked)
     ui->mainTabMenu->setCurrentIndex(checked ? 2 : 0);
 }
 
+void
+CallWidget::searchContactLineEditEntry(const URI &uri)
+{
+    auto cm = PhoneDirectoryModel::instance().getNumber(uri);
+    // if its a new CM, bring it to the top
+    if (cm->lastUsed() == 0)
+        cm->setLastUsed(QDateTime::currentDateTime().toTime_t());
+
+    // select cm
+    RecentModel::instance().selectionModel()->setCurrentIndex(RecentModel::instance().getIndex(cm),
+                                                              QItemSelectionModel::ClearAndSelect);
+    ui->ringContactLineEdit->clear();
+}
+
 void
 CallWidget::on_ringContactLineEdit_returnPressed()
 {
-    auto cm = PhoneDirectoryModel::instance().getNumber(ui->ringContactLineEdit->text());
-    time_t currentTime;
-    ::time(&currentTime);
-    cm->setLastUsed(currentTime);
-    ui->ringContactLineEdit->clear();
+    bool lookup_username = false;
+    auto contactLineText = ui->ringContactLineEdit->text();
+    URI uri_passed = URI(contactLineText);
 
-    QModelIndex index = RecentModel::instance().getIndex(cm);
-    RecentModel::instance().selectionModel()->setCurrentIndex(index, QItemSelectionModel::ClearAndSelect);
+    if (!contactLineText.isNull() && !contactLineText.isEmpty()){
+        if (uri_passed.protocolHint() == URI::ProtocolHint::RING_USERNAME ) {
+            lookup_username = TRUE;
+        } else if (
+            uri_passed.protocolHint() != URI::ProtocolHint::RING && // not a RingID
+            uri_passed.schemeType() == URI::SchemeType::NONE // scheme type not specified
+        ){
+            // if no scheme type has been specified, determine ring vs sip by the first available account
+            auto idx = AvailableAccountModel::instance().index(0, 0);
+            if (idx.isValid()) {
+                auto account = idx.data((int)Ring::Role::Object).value<Account *>();
+                if (account && account->protocol() == Account::Protocol::RING)
+                    lookup_username = TRUE;
+            }
+        }
+
+        if (lookup_username){
+            NameDirectory::instance().lookupName(nullptr, QString(), uri_passed);
+        } else {
+            searchContactLineEditEntry(uri_passed);
+        }
+    }
 }
 
 void
@@ -816,3 +866,38 @@ CallWidget::on_shareButton_clicked()
 {
     Utils::InvokeMailto(tr("Contact me on Ring"), tr("My RingId is : ") + ui->ringIdLabel->text());
 }
+
+void
+CallWidget::contactLineEdit_registeredNameFound(const Account* account,NameDirectory::LookupStatus status,
+                                                const QString& address,const QString& name)
+{
+    URI uri = URI(ui->ringContactLineEdit->text());
+    QString username_to_lookup = uri.userinfo();
+
+    if (username_to_lookup.compare(name) != 0){
+        return;
+    }
+
+    switch (status)
+    {
+        case NameDirectory::LookupStatus::SUCCESS:
+        {
+            uri = URI("ring:" + address);
+            qDebug() << uri;
+            searchContactLineEditEntry(uri);
+            break;
+        }
+        case NameDirectory::LookupStatus::INVALID_NAME:
+        {
+           qDebug() << "Invalid Ring username";
+           break;
+        }
+        case NameDirectory::LookupStatus::ERROR:
+        case NameDirectory::LookupStatus::NOT_FOUND:
+        default:
+        {
+            qDebug() << "Could not resolve Ring username";
+            break;
+        }
+    }
+}
